@@ -227,8 +227,6 @@ let spawn_dot graph status done_cb =
 	     "set -o pipefail ; \
               tee agraph.in.dot | %s -q -y -s%.0f | tee agraph.out.dot" dot_prg ppi ]
     else [ dot_prg ; "-q" ; "-y" ; Printf.sprintf "-s%.0f" ppi ] in
-  if Viz_misc.debug "exec"
-  then Printf.eprintf "### exec: Running '%s'\n%!" (String.concat " " cmd) ;
   let error fmt =
     Printf.kprintf (fun s -> done_cb (`LAYOUT_ERROR s)) fmt in
   try
