@@ -24,7 +24,7 @@ external new_file : string -> string -> t = "ml_g_io_channel_new_file"
 
 external read_chars : t -> ?off:int -> ?len:int -> string -> 
   [ `NORMAL of int | `AGAIN | `EOF]
-  = "ml_g_io_channel_read_chars"
+  = "_ml_g_io_channel_read_chars"
 external write_chars : t -> ?bytes_written:int ref -> ?off:int -> ?len:int -> string -> 
   [ `NORMAL of int | `AGAIN]
   = "ml_g_io_channel_write_chars"
