@@ -134,7 +134,7 @@ check_substring (value s, gsize off, gsize len)
 #endif
 
 CAMLprim value
-ml_g_io_channel_read_chars (value c, value o_off, value o_len, value s)
+_ml_g_io_channel_read_chars (value c, value o_off, value o_len, value s)
 {
   CAMLparam1(s);
   GError *err = NULL;
