@@ -6,24 +6,14 @@ type mtn_options =
   | MTNopt_branch of string * string
   | MTNopt_full of string * string * string
 
-let unquote s =
-  if s.[0] = '"'
-  then 
-    let len = String.length s in
-    Revision_lexer.string 
-      (Buffer.create len)
-      (Lexing.from_string (String.sub s 1 (len - 1)))
-  else s
-
 let find_MTN_dir base =
-  let rec up = function
-    | "/" -> raise Not_found
-    | p ->
-	let d = Filename.dirname p in
-	let m = Filename.concat d base in
-	if Sys.file_exists m 
-	then m
-	else up d in
+  let rec up p =
+    let d = Filename.dirname p in
+    if d = p then raise Not_found ;
+    let m = Filename.concat d base in
+    if Sys.file_exists m 
+    then m
+    else up d in
   if Sys.file_exists base
   then base
   else up (Sys.getcwd ())
@@ -34,17 +24,15 @@ let find_MTN_dir () =
 
 
 let parse_MTN_options mtn_file =
-  let lines =
-    try with_file_in input_lines (mtn_file "options")
+  let stanzas =
+    try 
+      with_file_in 
+	(fun ic -> Basic_io_lexer.parse (Lexing.from_channel ic))
+	(mtn_file "options")
     with Not_found | Sys_error _ -> [] in
-  try
-    List.fold_right
-      (fun s acc ->
-	match string_split ~max_elem:2 ' ' s with
-	| [a; b] -> (a, unquote b) :: acc
-	| _ -> acc)
-      lines [] 
-  with Failure _ -> []
+  List.map
+    (fun (k, v) -> k, Basic_io_lexer.string_of_elem v)
+    (List.flatten stanzas)
 
 let parse_MTN_revision mtn_file = 
   let format =
@@ -62,11 +50,10 @@ let parse_MTN_revision mtn_file =
       with_file_in 
 	(fun ic ->
 	  match 
-	    Revision_parser.revision_set
-	      Revision_lexer.lex 
+	    Revision.revision_set
 	      (Lexing.from_channel ic)
 	  with
-	  | _, { Revision_types.old_revision = r } :: _ -> r
+	  | _, { Revision.old_revision = r } :: _ -> r
 	  | _ -> failwith "could not determine revision id from _MTN/revision")
 	rev_file
   | _ ->
@@ -117,7 +104,7 @@ let parse_options args =
 
 let print_version () =
   Printf.printf "monotone-viz %s (base revision: %s)\n" Version.version Version.revision ;
-  Printf.printf "Copyright (C) 2004-2005 Olivier Andrieu <oandrieu@nerim.net>\n" ;
+  Printf.printf "Copyright (C) 2004-2006 Olivier Andrieu <oandrieu@gmail.com>\n" ;
   exit 0
 
 let parse_cli () =
@@ -134,8 +121,6 @@ let parse_cli () =
 
 
 let exn_handler ctrl = function
-  | Sqlite3.Error ((Sqlite3.BUSY | Sqlite3.LOCKED), _) ->
-      ()
   | exn ->
       ctrl#error_notice 
 	begin 
