@@ -1,46 +1,20 @@
+open Viz_misc
 open Viz_types
 
 module Selector = struct
 
-  let escape_selector s =
-    let len = String.length s in
-    let nb_escp = ref 0 in
-    for i = 0 to len - 1 do
-      let c = s.[i] in
-      if c = '\\' || c = '/'
-      then incr nb_escp
-    done ;
-    if !nb_escp = 0
-    then s
-    else begin
-      let o = String.create (len + !nb_escp) in
-      let j = ref 0 in
-      for i = 0 to len - 1 do
-	let c = s.[i] in
-	if c = '\\' || c = '/'
-	then (o.[!j] <- '\\' ; incr j) ;
-	o.[!j] <- c ;
-	incr j
-      done ;
-      assert (!j = len + !nb_escp) ;
-      o
-    end
-
   let make_selectors g sel =
-    match (Agraph.get_query g).dom with
-    | QUERY_ALL -> 
-	[ escape_selector sel ]
-    | QUERY_BRANCHES br -> 
-	List.map
-	  (fun b -> Printf.sprintf "b:%s/%s" (escape_selector b) (escape_selector sel))
-	  br
+    let br = (Agraph.get_query g).dom in
+    List.map
+      (fun b -> Printf.sprintf "b:%s/%s" (Monotone.escape_selector b) sel)
+      br
 
   let running_select = ref None
 
   let abort () =
     match !running_select with
-    | Some id -> 
-	Subprocess.abort id ;
+    | Some (mtn, id) -> 
+	List.iter (Automate.abort mtn) id ;
 	running_select := None
     | _ -> 
 	()
@@ -53,17 +27,15 @@ module Selector = struct
 	`IDS (List.filter (Agraph.mem g) ids)
     | x -> x
 
-  let select ctrl db g sel cont =
-    let id =
-      Database.run_monotone_select
-	db
-	ctrl#get_prefs.Viz_style.monotone_path
-	(ctrl#status "search")
+  let select mtn g sel cont =
+    let ids =
+      Monotone.select_async
+	mtn
 	(fun r ->
 	  running_select := None ;
 	  cont (filter_present g r))
 	(make_selectors g sel) in
-    running_select := Some id
+    running_select := Some (mtn, ids)
 
 end
 
@@ -71,16 +43,14 @@ let revision_contains pat = function
   | [ _, changes ] ->
       List.exists
 	(function
-	  | Revision_types.PATCH (f, _, _)
-	  | Revision_types.ADD_FILE (f, _)
-	  | Revision_types.ADD_DIR f
-	  | Revision_types.DELETE_FILE f
-	  | Revision_types.DELETE_DIR f
-	  | Revision_types.ATTR_CLEAR (_, f)
-	  | Revision_types.ATTR_SET (_, f, _) ->
+	  | Revision.PATCH (f, _, _)
+	  | Revision.ADD_FILE (f, _)
+	  | Revision.ADD_DIR f
+	  | Revision.DELETE f
+	  | Revision.ATTR_CLEAR (_, f)
+	  | Revision.ATTR_SET (_, f, _) ->
 	      Gpattern.match_string pat f
-	  | Revision_types.RENAME_FILE (f1, f2)
-	  | Revision_types.RENAME_DIR (f1, f2) ->
+	  | Revision.RENAME (f1, f2) ->
 	      Gpattern.match_string pat f1 ||
 	      Gpattern.match_string pat f2)
 	changes
@@ -90,30 +60,32 @@ let revision_contains pat = function
 
 let filter_by_revision_content 
     (ctrl : <status : string -> <with_status : 'a. string -> (unit -> 'a) -> 'a; ..>; ..>) 
-    db revision_content ids =
+    mtn revision_content ids =
   (ctrl#status "search")#with_status
     "Searching the monotone database ..."
     (fun () ->
       let pat = Gpattern.make revision_content in
       Ui.fold_in_loop
 	(fun acc id ->
-	  let r = Database.fetch_revision db id in
+	  let r = Monotone.get_revision mtn id in
 	  if revision_contains pat r.revision_set
 	  then id :: acc
 	  else acc)
 	[] ids)
 
-let select_by_revision_content ctrl db revision_content g =
+let select_by_revision_content ctrl mtn revision_content g =
   filter_by_revision_content 
-    ctrl db revision_content
+    ctrl mtn revision_content
     (Agraph.get_ids g)
 
 
-let expand_results db ids =
-  let fetch_first_cert id c =
-    match Database.fetch_cert_value db id c with
-    | h :: _ -> h
-    | [] -> "" in
+let expand_results mtn ids =
+  let fetch_first_cert id =
+    let get_cert = Monotone.cert_value mtn id in
+    fun c ->
+      match get_cert c with
+      | h :: _ -> h
+      | [] -> "" in
   
   List.map
     (fun id ->
@@ -126,26 +98,27 @@ let expand_results db ids =
 let do_query ~selector ~revision_content ctrl results_cb =
   let no_results () =
     results_cb (`IDS []) in
-  let results_ids db ids =
-    results_cb (`IDS (expand_results db ids)) in
+  let results_ids mtn ids =
+    results_cb (`IDS (expand_results mtn ids)) in
 
-  match ctrl#get_db, ctrl#get_agraph with
-  | Some db, Some g when selector <> "" ->
+  match ctrl#get_mtn, ctrl#get_agraph with
+  | Some mtn, Some g when selector <> "" ->
       Selector.select 
-	ctrl db g selector 
+	mtn g selector 
 	(function
 	  | `IDS ids when revision_content <> "" ->
-	      results_ids db
+	      results_ids 
+		mtn
 		(filter_by_revision_content 
-		   ctrl db revision_content ids)
+		   ctrl mtn revision_content ids)
 	  | `IDS ids ->
-	      results_ids db ids
+	      results_ids mtn ids
 	  | `SUB_PROC_ERROR _ as err -> 
 	      results_cb err)
-  | Some db, Some g when revision_content <> "" ->
-      results_ids db
+  | Some mtn, Some g when revision_content <> "" ->
+      results_ids mtn
 	(select_by_revision_content 
-	   ctrl db revision_content g)
+	   ctrl mtn revision_content g)
   | _ ->
       no_results ()
   
