@@ -32,9 +32,6 @@ let with_grab f =
 let pump () =
   while Glib.Main.iteration false do () done
 
-let nice_fetch f db =
-  with_grab (fun () -> Database.with_progress pump f db)
-
 let fold_in_loop ?(granularity=10) f init l =
   with_grab (fun () ->
     let i = ref 0 in
@@ -208,7 +205,6 @@ module Prefs = struct
 	     if group = None then Some b#group else group)
 	   None
 	   [ "no automatic coloring", NONE ;
-	     "color by key_id",       BY_KEYID ;
 	     "color by author",       BY_AUTHOR_HASH ;
 	     "color by branch",       BY_BRANCH_HASH ] )
     end ;
@@ -265,7 +261,38 @@ module Prefs = struct
 
 end
 
+module About = struct
 
+  let authors  = ["Olivier Andrieu <oandrieu@gmail.com>"]
+  let comments = "Lets you visualize ancestry graphs from the Revision Control System monotone"
+  let copyright = "Copyright © 2004-2007 Olivier Andrieu"
+  let license   = "\
+monotone-viz is free software; you can redistribute it 
+and/or modify it under the terms of the GNU General 
+Public License as published by the Free Software Foundation; 
+either version 2 of the License, or (at your option) 
+any later version."
+
+  let name = "monotone-viz"
+  let version = Version.version
+  let website = "http://oandrieu.nerim.net/monotone-viz/"
+  let website_label = "monotone-viz website"
+
+  let make ctrl =
+    let d =
+      GWindow.about_dialog
+	~authors ~comments ~copyright
+	~license ~logo:(Lazy.force Icon.monotone)
+	~name ~version ~website ~website_label
+	~parent:ctrl#get_toplevel () in
+    ignore (d#event#connect#delete (fun _ -> d#misc#hide () ; true)) ;
+    ignore (d#connect#response (fun _ -> d#misc#hide ())) ;
+    d
+
+  let show ctrl =
+    let d = lazy (make ctrl) in
+    fun () -> (Lazy.force d)#present ()
+end
 
 module Open = struct
   type t = [`OPEN|`CLOSE|`DELETE_EVENT] GWindow.file_chooser_dialog 
@@ -288,6 +315,7 @@ module Open = struct
     resp
 end
 
+(*
 module LockedDB = struct
 
   let message ctrl =
@@ -325,6 +353,7 @@ module LockedDB = struct
 		())
     end ;
     begin
+      ignore (dialog#connect#close (fun () -> dialog#response `CANCEL)) ;
       dialog#add_button_stock `CANCEL `CANCEL ;
       dialog#add_button "Retry" `RETRY
     end ;
@@ -336,7 +365,7 @@ module LockedDB = struct
     resp
 
 end
-
+*)
   
 
 
@@ -374,6 +403,9 @@ let ui_info = "\
         <menuitem action='Zoom_out'/>\
         <menuitem action='Query'/>\
       </menu>\
+      <menu action='HelpMenu'>\
+        <menuitem action='About'/>\
+      </menu>\
     </menubar>\
     <accelerator action='FindEntry'/>\
   </ui>"
@@ -404,7 +436,9 @@ let make_groups () =
   add "Open"  ~stock:`OPEN  ~tooltip:"Open a database" ;
   add "Quit"  ~stock:`QUIT  ~tooltip:"Exit" ;
   add "Prefs" ~stock:`PREFERENCES ~tooltip:"Edit Preferences" ;
-  add "FindEntry" ~accel:"<Ctrl>l" ] ;
+  add "FindEntry" ~accel:"<Ctrl>l" ;
+  add "HelpMenu" ~label:"_Help" ;
+  add "About" ~stock:`ABOUT ] ;
   let g_db = GAction.action_group ~name:"db" () in
   GAction.add_actions g_db [
   add "Close" ~stock:`CLOSE ~tooltip:"Close the database" ;
@@ -489,7 +523,7 @@ let popup m ctrl ~popup_id button =
   begin
     let copy_revision = p.group#get_action "Copy_revision" in
     let copy_manifest = p.group#get_action "Copy_manifest" in
-    let data = Database.fetch_revision (some ctrl#get_db) popup_id in
+    let data = Monotone.get_revision (some ctrl#get_mtn) popup_id in
     remember_signal copy_revision
       (fun () -> set_clipboard m data.revision_id) ;
     remember_signal copy_manifest
@@ -541,6 +575,25 @@ let popup m ctrl ~popup_id button =
 
 
 
+let automate_cb auto o =
+  begin 
+    match o with
+    | `OUTPUT msg ->
+	Printf.eprintf "### automate: output '%s'\n%!" (String.escaped msg) ;
+	let message = Printf.sprintf "<tt><b>interface_version</b>: %s</tt>" msg in
+	let d = 
+	  GWindow.message_dialog
+	    ~message ~use_markup:true
+	    ~message_type:`INFO
+	    ~buttons:GWindow.Buttons.close () in
+	ignore (d#run ()) ;
+	d#destroy ()
+    | `SYNTAX_ERROR msg ->
+	Printf.eprintf "### automate: syntax error '%s'\n%!" msg
+    | `ERROR msg ->
+	Printf.eprintf "### automate: error '%s'\n%!" msg
+  end ;
+  ignore (Glib.Timeout.add 5000 (fun () -> Automate.exit auto ; false))
 
 
 
@@ -570,6 +623,7 @@ let setup ({ manager = ui } as m) ctrl =
   action_connect "/menubar/FileMenu/Prefs"    ctrl#show_prefs ;
   action_connect "/menubar/ViewMenu/Query"    ctrl#show_search ;
   action_connect "/menubar/FileMenu/New"      ctrl#show_view ;
+  action_connect "/menubar/HelpMenu/About"    (About.show ctrl) ;
   action_connect "/popup/Certs"      
     (fun () -> ctrl#display_certs (get_popup_data m).popup_id) ;
   action_connect "/popup_cert/Copy_cert"
@@ -577,7 +631,7 @@ let setup ({ manager = ui } as m) ctrl =
       may
 	(set_clipboard m) 
 	ctrl#get_current_cert_value) ;
-  action_connect "/FindEntry" ctrl#focus_find_entry 
+  action_connect "/FindEntry" ctrl#focus_find_entry
 
 
 let open_db m ctrl =
