@@ -28,7 +28,7 @@ type text_props = [
 val match_style : 
   prefs -> 
   Agraph.t -> 
-  Database.t ->
+  Monotone.t ->
   string -> 
   shape_props list * text_props list -> 
   [> shape_props | `FILL_COLOR_RGBA of int32] list * [> text_props] list
