@@ -5,9 +5,7 @@ let errorf fmt = Printf.kprintf error fmt
 
 module StringMap = Map.Make(String)
 
-type query_domain =
-  | QUERY_ALL
-  | QUERY_BRANCHES of string list
+type query_domain = string list
 type date = string
 type query_limit =
   | QUERY_NO_LIMIT
@@ -64,7 +62,7 @@ let neighbour_kind = function
 type node_data = {
     revision_id  : string ;
     manifest_id  : string ;
-    revision_set : (string * Revision_types.change list) list ;
+    revision_set : (string * Revision.change list) list ;
     certs        : cert list ;
   }
 
@@ -115,6 +113,5 @@ type direction = [
 
 type autocolor =
   | NONE
-  | BY_KEYID
   | BY_AUTHOR_HASH
   | BY_BRANCH_HASH
