@@ -7,9 +7,7 @@ val errorf : ('a, unit, string, 'b) format4 -> 'a
 
 module StringMap : Map.S with type key = string
 
-type query_domain =
-  | QUERY_ALL
-  | QUERY_BRANCHES of string list
+type query_domain = string list
 type date = string
 type query_limit =
   | QUERY_NO_LIMIT
@@ -61,7 +59,7 @@ val neighbour_kind : node_kind -> bool
 type node_data = {
     revision_id  : string ;
     manifest_id  : string ;
-    revision_set : (string * Revision_types.change list) list ;
+    revision_set : (string * Revision.change list) list ;
     certs        : cert list ;
   }
 
@@ -115,6 +113,5 @@ type direction = [
 
 type autocolor =
   | NONE
-  | BY_KEYID
   | BY_AUTHOR_HASH
   | BY_BRANCH_HASH
